import bisect


def chunk(iterator, count):
    itr = iter(iterator)
    while True:
        try:
            yield tuple([next(itr) for i in range(count)])
        except StopIteration:
            return


def seek_patch_data(patches, offset, num_bytes):
    offsetlist = []
    for patch in patches:
        for key, value in patch.items():
            offsetlist.append(int(key))
    offsetlist_sorted = sorted(offsetlist)
    i = bisect.bisect_left(offsetlist_sorted, offset)
    if i:
        if offsetlist_sorted[i] == offset:
            seek = str(offset)
            for patch in patches:
                if seek in patch:
                    return patch[seek][:num_bytes]
        else:
            left_slice = offset - offsetlist_sorted[i - 1]
            for patch in patches:
                seek = str(offsetlist_sorted[i - 1])
                if seek in patch:
                    return patch[seek][left_slice:left_slice + num_bytes]
    raise ValueError



# far from done
from . import exceptions

import aiohttp
import json


class site():
    def __init__(
            self,
            site_baseurl,
            patch_baseurl=None,
            username=None,
            password=None):
        self.site_baseurl = site_baseurl
        self.patch_baseurl = patch_baseurl

        self.auth = aiohttp.BasicAuth(login=username, password=password) if username else None

    async def generate_game(self, endpoint, settings):
        for i in range(0, 5):
            try:
                req = await request_json_post(
                    url=self.site_baseurl + endpoint,
                    data=settings,
                    auth=self.auth,
                    returntype='json'
                )
                return req
            except aiohttp.client_exceptions.ServerDisconnectedError:
                continue
            # except aiohttp.ClientResponseError:
            #     continue
        raise exceptions.alttprFailedToGenerate('failed to generate game')

    async def retrieve_game(self, hash_id):
        for i in range(0, 5):
            try:
                if self.patch_baseurl is not None:
                    s3patch = await request_generic(
                        url=self.patch_baseurl + '/' + hash_id + '.json',
                        returntype='json'
                    )
                    return s3patch
                else:
                    localpatch = await request_generic(
                        url=self.site_baseurl + '/hash/' + hash_id,
                        auth=self.auth,
                        returntype='json'
                    )
                    return localpatch
            except aiohttp.ClientResponseError:
                localpatch = await request_generic(
                    url=self.site_baseurl + '/hash/' + hash_id,
                    auth=self.auth,
                    returntype='json'
                )
                # if localpatch.status == 404:
                #     break
                return localpatch
            except aiohttp.client_exceptions.ServerDisconnectedError:
                continue
        raise exceptions.alttprFailedToRetrieve(
            'failed to retrieve game {hash}, the game is likely not found'.format(
                hash=hash_id))

    async def retrieve_json(self, endpoint, useauth=True):
        if useauth:
            auth = self.auth
        else:
            auth = None

        req = await request_generic(
            url=self.site_baseurl + endpoint,
            auth=auth,
            returntype='json'
        )
        return req

    async def retrieve_url_raw_content(self, url, useauth=True):
        if useauth:
            auth = self.auth
        else:
            auth = None

        req = await request_generic(
            url=url,
            auth=auth,
            returntype='binary'
        )
        return req


async def request_generic(url, method='get', reqparams=None, data=None, header=None, auth=None, returntype='text'):
    async with aiohttp.ClientSession(auth=None, raise_for_status=True) as session:
        async with session.request(method.upper(), url, params=reqparams, data=data, headers=header, auth=auth) as resp:
            if returntype == 'text':
                return await resp.text()
            elif returntype == 'json':
                return json.loads(await resp.text())
            elif returntype == 'binary':
                return await resp.read()

async def request_json_put(url, data, auth=None, returntype='text'):
    async with aiohttp.ClientSession(auth=auth, raise_for_status=True) as session:
        async with session.put(url=url, json=data, auth=auth) as resp:
            if returntype == 'text':
                return await resp.text()
            elif returntype == 'json':
                return json.loads(await resp.text())
            elif returntype == 'binary':
                return await resp.read()

async def request_json_post(url, data, auth=None, returntype='text'):
    async with aiohttp.ClientSession(auth=auth, raise_for_status=True) as session:
        async with session.post(url=url, json=data, auth=auth) as resp:
            if returntype == 'text':
                return await resp.text()
            elif returntype == 'json':
                return json.loads(await resp.text())
            elif returntype == 'binary':
                return await resp.read()



from . import http
import slugid
import uuid


async def sm(
    settings=None,
    slug_id=None,
    guid_id=None,
    baseurl='https://beta.samus.link',
    randomizer='sm',
    username=None,
    password=None,
):
    seed = smClass(
        settings=settings,
        slug_id=slug_id,
        guid_id=guid_id,
        baseurl=baseurl,
        randomizer=randomizer,
        username=username,
        password=password
    )
    await seed._init()
    return seed


class smClass():
    def __init__(
        self,
        settings,
        slug_id,
        guid_id,
        baseurl,
        randomizer,
        username,
        password,
    ):
        self.settings = settings
        self.slug_id = slug_id
        self.guid_id = guid_id
        self.baseurl = baseurl
        self.randomizer = randomizer
        self.username = username
        self.password = password

    async def _init(self):
        self.site = http.site(
            site_baseurl=self.baseurl,
            username=self.username,
            password=self.password,
        )

        if self.settings:
            endpoint = f'/api/randomizers/{self.randomizer}/generate'
            self.data = await self.site.generate_game(endpoint, self.settings)
            self.guid = uuid.UUID(hex=self.data['guid'])
            self.slug_id = slugid.encode(self.guid)
        elif self.slug_id:
            self.guid = slugid.decode(self.slug_id)
            self.data = await http.request_generic(
                url=f'{self.baseurl}/api/seed/{self.guid.hex}',
                method='get',
                returntype='json'
            )
        elif self.guid_id:
            self.guid = uuid.UUID(hex=self.guid_id)
            self.slug_id = slugid.encode(self.guid)
            self.data = await http.request_generic(
                url=f'{self.baseurl}/api/seed/{self.guid.hex}',
                method='get',
                returntype='json'
            )
        else:
            self.data = None
            self.slug_id = None
            self.guid = None

        self.url = f'{self.baseurl}/seed/{self.slug_id}'

    async def randomizer_settings(self):
        """Returns a dictonary of valid settings.

        Returns:
            dict -- dictonary of valid settings that can be used
        """
        return await http.request_generic(
            url=f'/api/randomizers/{self.randomizer}',
            method='get',
            returntype='json'
        )




import copy
import random
from .customizer import get_starting_equipment, BASE_CUSTOMIZER_PAYLOAD

BASE_RANDOMIZER_PAYLOAD = {
    "glitches": "none",
    "item_placement": "advanced",
    "dungeon_items": "standard",
    "accessibility": "items",
    "goal": "ganon",
    "crystals": {
        "ganon": "7",
        "tower": "7"
    },
    "mode": "open",
    "entrances": "none",
    "hints": "off",
    "weapons": "randomized",
    "item": {
        "pool": "normal",
        "functionality": "normal"
    },
    "tournament": False,
    "spoilers": "on",
    "lang": "en",
    "enemizer": {
        "boss_shuffle": "none",
        "enemy_shuffle": "none",
        "enemy_damage": "default",
        "enemy_health": "default"
    }
}

def generate_random_settings(weights, tournament=True, spoilers="mystery"):
    # customizer isn't used until its used
    customizer = False

    # we need to figure out if entrance shuffle is a thing, since that tells us if we should even bother with rolling customizer things
    entrances =  get_random_option(weights['entrance_shuffle'])

    # only roll customizer stuff if entrance shuffle isn't on, and we have a customizer section
    if entrances == "none" and 'customizer' in weights:
        custom = {}
        eq = []
        pool = {}

        if 'eq' in weights['customizer']:
            for key in weights['customizer']['eq'].keys():
                value = get_random_option(weights['customizer']['eq'][key])
                if value is not None:
                    eq += get_starting_equipment(key=key, value=value)
                    customizer = True

        if 'custom' in weights['customizer']:
            for key in weights['customizer']['custom'].keys():
                value = get_random_option(weights['customizer']['custom'][key])
                if value is not None:
                    custom[key] = value
                    customizer = True

        if 'pool' in weights['pool']:
            for key in weights['customizer']['pool'].keys():
                value = get_random_option(weights['customizer']['pool'][key])
                if value is not None:
                    pool[key] = value
                    customizer = True

    if customizer:
        settings = copy.deepcopy(BASE_CUSTOMIZER_PAYLOAD)
    else:
        settings = copy.deepcopy(BASE_RANDOMIZER_PAYLOAD)
    
    settings["glitches"] = get_random_option(weights['glitches_required'])
    settings["item_placement"] = get_random_option(weights['item_placement'])
    settings["dungeon_items"] = get_random_option(weights['dungeon_items'])
    settings["accessibility"] = get_random_option(weights['accessibility'])
    settings["goal"] = get_random_option(weights['goals'])
    settings["crystals"]["ganon"] = get_random_option(weights['tower_open'])
    settings["crystals"]["tower"] = get_random_option(weights['ganon_open'])
    settings["mode"] = get_random_option(weights['world_state'])
    settings["hints"] = get_random_option(weights['hints'])
    settings["weapons"] = get_random_option(weights['weapons'])
    settings["item"]["pool"] = get_random_option(weights['item_pool'])
    settings["item"]["functionality"] = get_random_option(weights['item_functionality'])
    settings["tournament"] = tournament
    settings["spoilers"] = spoilers
    settings["enemizer"]["boss_shuffle"] = get_random_option(weights['boss_shuffle'])
    settings["enemizer"]["enemy_shuffle"] = get_random_option(weights['enemy_shuffle'])
    settings["enemizer"]["enemy_damage"] = get_random_option(weights['enemy_damage'])
    settings["enemizer"]["enemy_health"] = get_random_option(weights['enemy_health'])

    if customizer:
        # default to v31 prize packs
        settings['custom']['customPrizePacks'] = False

        # set custom settings that were rolled
        for key, value in custom.items():
            settings['custom'][key] = value

        # set custom item pool that was rolled
        for key, value in pool.items():
            settings['custom']['item']['count'][key] = value

        # apply custom starting equipment, and adjust the item pool accordingly
        if eq:
            # remove items from pool
            for item in eq:
                settings['custom']['item']['count'][item] = settings['custom']['item']['count'].get(item, 0) - 1 if settings['custom']['item']['count'].get(item, 0) > 0 else 0

            # re-add 3 heart containers as a baseline
            eq += ['BossHeartContainer'] * 3

            # update the eq section of the settings
            settings['eq'] = eq

        # if dark room navigation is enabled, then 
        # oh and yes item.require.Lamp is mixed around for whatever reason
        # False = dark room navigation isn't required
        if settings['custom'].get('item.require.Lamp', False):
            settings['enemizer']['enemy_shuffle'] = 'none'
            settings['enemizer']['enemy_damage'] = 'default'

        # set dungeon_items to standard if any region.wild* custom settings are present
        if any(key in ['region.wildKeys', 'region.wildBigKeys', 'region.wildCompasses', 'region.wildMaps'] for key in custom):
            settings['dungeon_items'] = 'standard'

        # set custom triforce hunt settings if TFH is the goal
        if settings['goal'] == 'triforce-hunt':
            min_difference = get_random_option(weights['customizer']['triforce-hunt'].get('min_difference', 0))
            try:
                goal_pieces = random.randint(weights['customizer']['triforce-hunt']['goal']['min'], weights['customizer']['triforce-hunt']['goal']['max'])
            except KeyError:
                goal_pieces = 20

            try:
                pool_pieces = random.randint(
                    weights['customizer']['triforce-hunt']['pool']['min'] if weights['customizer']['triforce-hunt']['pool']['min'] + min_difference > goal_pieces else goal_pieces + min_difference,
                    weights['customizer']['triforce-hunt']['pool']['max'],
                )
            except KeyError:
                pool_pieces = 30

            settings['custom']['item.Goal.Required'] = goal_pieces
            settings['custom']['item']['count']['TriforcePiece'] = pool_pieces

    else:
        settings["entrances"] = entrances

    # This if statement is dedicated to the survivors of http://www.speedrunslive.com/races/result/#!/264658
    if settings['weapons'] not in ['vanilla', 'assured'] and settings['mode'] == 'standard' and (
            settings['enemizer']['enemy_shuffle'] != 'none'
            or settings['enemizer']['enemy_damage'] != 'default'
            or settings['enemizer']['enemy_health'] != 'default'):
        settings['weapons'] = 'assured'

    # Stop gap measure until swordless entrance with a hard+ item pool is fixed
    if settings.get('entrances', 'none') != 'none' and settings['item']['pool'] in ['hard', 'expert'] and settings['weapons'] == 'swordless':
        settings['weapons'] = 'randomized'

    return settings, customizer

def get_random_option(optset):
    return random.choices(population=list(optset.keys()), weights=list(optset.values()))[0] if isinstance(optset, dict) else optset



import math

BASE_CUSTOMIZER_PAYLOAD = {
    "glitches": "none",
    "item_placement": "advanced",
    "dungeon_items": "standard",
    "accessibility": "items",
    "goal": "ganon",
    "crystals": {
        "ganon": "7",
        "tower": "7"
    },
    "mode": "open",
    "weapons": "randomized",
    "hints": "on",
    "item": {
        "pool": "normal",
        "functionality": "normal"
    },
    "tournament": "false",
    "spoilers": "on",
    "lang": "en",
    "enemizer": {
        "boss_shuffle": "none",
        "enemy_shuffle": "none",
        "enemy_damage": "default",
        "enemy_health": "default"
    },
    "name": "",
    "notes": "",
    "l": {},
    "eq": ["BossHeartContainer", "BossHeartContainer", "BossHeartContainer"],
    "drops": {
        "0": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "1": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "2": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "3": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "4": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "5": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "6": ["auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill", "auto_fill"],
        "pull": ["auto_fill", "auto_fill", "auto_fill"],
        "crab": ["auto_fill", "auto_fill"],
        "stun": ["auto_fill"],
        "fish": ["auto_fill"]
    },
    "custom": {
        "item.Goal.Required": "",
        "item.require.Lamp": False,
        "item.value.BlueClock": "",
        "item.value.GreenClock": "",
        "item.value.RedClock": "",
        "item.value.Rupoor": "",
        "prize.crossWorld": True,
        "prize.shuffleCrystals": True,
        "prize.shufflePendants": True,
        "region.bossNormalLocation": True,
        "region.wildBigKeys": False,
        "region.wildCompasses": False,
        "region.wildKeys": False,
        "region.wildMaps": False,
        "rom.dungeonCount": "off",
        "rom.freeItemMenu": False,
        "rom.freeItemText": False,
        "rom.mapOnPickup": False,
        "rom.timerMode": "off",
        "rom.timerStart": "",
        "rom.rupeeBow": False,
        "rom.genericKeys": False,
        "spoil.BootsLocation": False,
        "canBombJump": False,
        "canBootsClip": False,
        "canBunnyRevive": False,
        "canBunnySurf": False,
        "canDungeonRevive": False,
        "canFakeFlipper": False,
        "canMirrorClip": False,
        "canMirrorWrap": False,
        "canOneFrameClipOW": False,
        "canOneFrameClipUW": False,
        "canOWYBA": False,
        "canSuperBunny": False,
        "canSuperSpeed": False,
        "canWaterFairyRevive": False,
        "canWaterWalk": False,
        "item": {
            "count": {
                "BottleWithRandom": 4,
                "Nothing": 0,
                "L1Sword": 0,
                "L1SwordAndShield": 0,
                "MasterSword": 0,
                "L3Sword": 0,
                "L4Sword": 0,
                "BlueShield": 0,
                "RedShield": 0,
                "MirrorShield": 0,
                "FireRod": 1,
                "IceRod": 1,
                "Hammer": 1,
                "Hookshot": 1,
                "Bow": 0,
                "Boomerang": 1,
                "Powder": 1,
                "Bombos": 1,
                "Ether": 1,
                "Quake": 1,
                "Lamp": 1,
                "Shovel": 1,
                "OcarinaInactive": 1,
                "CaneOfSomaria": 1,
                "Bottle": 0,
                "PieceOfHeart": 24,
                "CaneOfByrna": 1,
                "Cape": 1,
                "MagicMirror": 1,
                "PowerGlove": 0,
                "TitansMitt": 0,
                "BookOfMudora": 1,
                "Flippers": 1,
                "MoonPearl": 1,
                "BugCatchingNet": 1,
                "BlueMail": 0,
                "RedMail": 0,
                "Bomb": 0,
                "ThreeBombs": 16,
                "Mushroom": 1,
                "RedBoomerang": 1,
                "BottleWithRedPotion": 0,
                "BottleWithGreenPotion": 0,
                "BottleWithBluePotion": 0,
                "TenBombs": 1,
                "OneRupee": 2,
                "FiveRupees": 4,
                "TwentyRupees": 28,
                "BowAndArrows": 0,
                "BowAndSilverArrows": 0,
                "BottleWithBee": 0,
                "BottleWithFairy": 0,
                "BossHeartContainer": 10,
                "HeartContainer": 1,
                "OneHundredRupees": 1,
                "FiftyRupees": 7,
                "Heart": 0,
                "Arrow": 1,
                "TenArrows": 12,
                "SmallMagic": 0,
                "ThreeHundredRupees": 5,
                "BottleWithGoldBee": 0,
                "OcarinaActive": 0,
                "PegasusBoots": 1,
                "BombUpgrade5": 0,
                "BombUpgrade10": 0,
                "ArrowUpgrade5": 0,
                "ArrowUpgrade10": 0,
                "HalfMagic": 1,
                "QuarterMagic": 0,
                "SilverArrowUpgrade": 0,
                "Rupoor": 0,
                "RedClock": 0,
                "BlueClock": 0,
                "GreenClock": 0,
                "ProgressiveSword": 4,
                "ProgressiveShield": 3,
                "ProgressiveArmor": 2,
                "ProgressiveGlove": 2,
                "ProgressiveBow": 2,
                "Triforce": 0,
                "TriforcePiece": 0,
                "MapA2": 1,
                "MapD7": 1,
                "MapD4": 1,
                "MapP3": 1,
                "MapD5": 1,
                "MapD3": 1,
                "MapD6": 1,
                "MapD1": 1,
                "MapD2": 1,
                "MapA1": 0,
                "MapP2": 1,
                "MapP1": 1,
                "MapH1": 0,
                "MapH2": 1,
                "CompassA2": 1,
                "CompassD7": 1,
                "CompassD4": 1,
                "CompassP3": 1,
                "CompassD5": 1,
                "CompassD3": 1,
                "CompassD6": 1,
                "CompassD1": 1,
                "CompassD2": 1,
                "CompassA1": 0,
                "CompassP2": 1,
                "CompassP1": 1,
                "CompassH1": 0,
                "CompassH2": 0,
                "BigKeyA2": 1,
                "BigKeyD7": 1,
                "BigKeyD4": 1,
                "BigKeyP3": 1,
                "BigKeyD5": 1,
                "BigKeyD3": 1,
                "BigKeyD6": 1,
                "BigKeyD1": 1,
                "BigKeyD2": 1,
                "BigKeyA1": 0,
                "BigKeyP2": 1,
                "BigKeyP1": 1,
                "BigKeyH1": 0,
                "BigKeyH2": 0,
                "KeyH2": 1,
                "KeyH1": 0,
                "KeyP1": 0,
                "KeyP2": 1,
                "KeyA1": 2,
                "KeyD2": 1,
                "KeyD1": 6,
                "KeyD6": 3,
                "KeyD3": 3,
                "KeyD5": 2,
                "KeyP3": 1,
                "KeyD4": 1,
                "KeyD7": 4,
                "KeyA2": 4
            }
        },
        "drop": {
            "count": {
                "Bee": 0,
                "BeeGood": 0,
                "Heart": 13,
                "RupeeGreen": 9,
                "RupeeBlue": 7,
                "RupeeRed": 6,
                "BombRefill1": 7,
                "BombRefill4": 1,
                "BombRefill8": 2,
                "MagicRefillSmall": 6,
                "MagicRefillFull": 3,
                "ArrowRefill5": 5,
                "ArrowRefill10": 3,
                "Fairy": 1
            }
        }
    }
}

def convert2settings(
        customizer_save,
        tournament=False,
        spoilers="off",
        spoilers_ongen=False):
    """Converts a customizer-settings.json file from alttpr.com to a settings dictionary that can be used for generating a game.

    Arguments:
        customizer_save {dict} -- A dictionary of the customizer-settings.json file (needs to already be converted to a dict)

    Keyword Arguments:
        tournament {bool} -- Setting to True generates a race rom, which excludes the spoiler log. (default: {False})
        spoilers {str} -- Sets the spoiler mode. (default: {"off"})
        spoielrs_ongen {bool} -- Sets spoiler mode to "generate".  This is deprecated. (default: {False})

    Returns:
        dict -- a dictionary of settings that can be used with pyz3r.alttpr()
    """

    if spoilers_ongen:
        spoilers = "generate"

    # the settings defaults, hopefully this is accurate

    settings = BASE_CUSTOMIZER_PAYLOAD
    settings['tournament'] = tournament
    settings['spoilers'] = spoilers

    try:
        if not customizer_save['randomizer.glitches_required'] is None:
            settings['glitches'] = customizer_save['randomizer.glitches_required']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.accessibility'] is None:
            settings['accessibility'] = customizer_save['randomizer.accessibility']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.goal'] is None:
            settings['goal'] = customizer_save['randomizer.goal']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.tower_open'] is None:
            settings['crystals']['tower'] = customizer_save['randomizer.tower_open']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.ganon_open'] is None:
            settings['crystals']['ganon'] = customizer_save['randomizer.ganon_open']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.dungeon_items'] is None:
            settings['dungeon_items'] = customizer_save['randomizer.dungeon_items']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.item_placement'] is None:
            settings['item_placement'] = customizer_save['randomizer.item_placement']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.world_state'] is None:
            settings['mode'] = customizer_save['randomizer.world_state']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.hints'] is None:
            settings['hints'] = customizer_save['randomizer.hints']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.weapons'] is None:
            settings['weapons'] = customizer_save['randomizer.weapons']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.item_pool'] is None:
            settings['item']['pool'] = customizer_save['randomizer.item_pool']
    except KeyError:
        pass

    try:
        if not customizer_save['randomizer.item_functionality'] is None:
            settings['item']['functionality'] = customizer_save['randomizer.item_functionality']
    except KeyError:
        pass

    # vt.custom.prizepacks
    try:
        if not customizer_save['vt.custom.prizepacks'] is None:
            settings['drops'] = customizer_save['vt.custom.prizepacks']
    except KeyError:
        pass

    # vt.custom.drops
    try:
        if not customizer_save['vt.custom.drops'] is None:
            settings['drop']['count'] = customizer_save['vt.custom.drops']
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.settings'] is None:
            for key, value in customizer_save['vt.custom.settings'].items(
            ):
                settings['custom'][key] = value
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.glitches'] is None:
            for key, value in customizer_save['vt.custom.glitches'].items(
            ):
                settings['custom'][key] = value
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.items'] is None:
            settings['custom']['item']['count'] = customizer_save['vt.custom.items']
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.locations'] is None:
            settings['l'] = customizer_save['vt.custom.locations']
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.notes'] is None:
            settings['notes'] = customizer_save['vt.custom.notes']
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.name'] is None:
            settings['name'] = customizer_save['vt.custom.name']
    except KeyError:
        pass

    try:
        if not customizer_save['vt.custom.equipment'] is None:
            eq = []
            for key, value in customizer_save['vt.custom.equipment'].items():
                 eq += get_starting_equipment(key, value)

            settings['eq'] = eq

    except KeyError:
        pass

    return settings

def get_starting_equipment(key, value):
    eq = []
    if key in ['Bottle1', 'Bottle2', 'Bottle3', 'Bottle4']:
        if value == 1:
            eq += ['Bottle']
        elif value == 2:
            eq += ['BottleWithRedPotion']
        elif value == 3:
            eq += ['BottleWithGreenPotion']
        elif value == 4:
            eq += ['BottleWithBluePotion']
        elif value == 5:
            eq += ['BottleWithBee']
        elif value == 6:
            eq += ['BottleWithGoldBee']
        elif value == 7:
            eq += ['BottleWithFairy']
    elif key == 'ProgressiveBow':
        if value == 1:
            eq += ['SilverArrowUpgrade']
        elif value == 2:
            eq += ['Bow']
        elif value == 3:
            eq += ['BowAndSilverArrows']
    elif key == 'Boomerang':
        if value == 1:
            eq += ['Boomerang']
        elif value == 2:
            eq += ['RedBoomerang']
        elif value == 3:
            eq += ['Boomerang', 'RedBoomerang']
    elif key == 'Ocarina':
        if value == 1:
            eq += ['OcarinaInactive']
        elif value == 2:
            eq += ['OcarinaActive']
    elif key == "Rupees":
        value = int(value)

        eq += math.floor(value / 300) * ['ThreeHundredRupees']
        value %= 300

        eq += math.floor(value / 100) * ['OneHundredRupees']
        value %= 100

        eq += math.floor(value / 50) * ['FiftyRupees']
        value %= 50

        eq += math.floor(value / 20) * ['TwentyRupees']
        value %= 20

        eq += math.floor(value / 5) * ['FiveRupees']
        value %= 5

        eq += math.floor(value / 1) * ['OneRupee']
    else:
        eq += int(value) * [key]

    return eq


"""
ALTTPR API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the ALTTPR.com API.
"""

__title__ = 'pyz3r'
__author__ = 'Thomas Prescott'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2018-2019 Thomas Prescott'
__version__ = '4.5.5'

from .alttpr import alttpr
from .sm import sm
from .smz3 import smz3
from . import rom, mystery, customizer, patch
